package hapra.pong;

import java.util.ArrayList;
import java.util.Date;
import java.util.Random;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.view.MotionEvent;
import android.view.SurfaceHolder;
import android.view.SurfaceView;

/**
 * Spiellogik
 * @author Jan Bauerdick
 *
 */
public class Pong extends SurfaceView implements SurfaceHolder.Callback {//TODO

	private ArrayList<GameObject> elements = new ArrayList<GameObject>();
	private GameThread thread;
	private Context parentContext;
	private boolean gameActive;
	private int acceleration;

	public Pong(Context context) {
		super(context);
		getHolder().addCallback(this);
		gameActive = false;
		acceleration = 0;
		parentContext = context;
		thread = new GameThread(getHolder(), this);
	}

	public boolean isActive() {
		return gameActive;
	}

	public void setActive(boolean active) {
		gameActive = active;
	}

	public void initGame() {
		synchronized (elements) {
			Paddle paddle1 = new Paddle(BitmapFactory.decodeResource(
					getResources(), R.drawable.paddle), getWidth(),
					getHeight(), false);
			elements.add(paddle1);
			Paddle paddle2 = new Paddle(BitmapFactory.decodeResource(
					getResources(), R.drawable.paddle), getWidth(),
					getHeight(), true);
			elements.add(paddle2);
		}
		//TODO: instantiate Orientation

	}

	@Override
	public void onDraw(Canvas canvas) {
		synchronized (elements) {
			canvas.drawColor(Color.BLACK);
			Bitmap bitmap;
			Coordinates coords;
			for (GameObject element : elements) {
				bitmap = element.getGraphic();
				coords = element.getCoordinates();
				canvas.drawBitmap(bitmap, coords.getX(), coords.getY(), null);
			}
		}
	}

	@Override
	public boolean onTouchEvent(MotionEvent event) {
		synchronized (elements) {
			if (event.getAction() == MotionEvent.ACTION_DOWN) {
				if (!isActive()) {
					acceleration = 0;
					Random rnd = new Random(new Date().getTime());
					int x = rnd.nextInt(2) + 1;
					int y = rnd.nextInt(2) + 1;
					boolean right = rnd.nextBoolean();
					boolean down = rnd.nextBoolean();
					Ball b = new Ball(BitmapFactory.decodeResource(
							getResources(), R.drawable.whiteball));
					b.getCoordinates().setX(
							getWidth() / 2 - b.getGraphic().getWidth() / 2);
					b.getCoordinates().setY(
							getHeight() / 2 - b.getGraphic().getHeight() / 2);
					b.getSpeed().setX(x);
					b.getSpeed().setY(y);
					b.getSpeed().setXDirection(
							right ? Speed.X_DIRECTION_RIGHT
									: Speed.X_DIRECTION_LEFT);
					b.getSpeed().setYDirection(
							down ? Speed.Y_DIRECTION_DOWN
									: Speed.Y_DIRECTION_UP);
					b.getSpeed().setYDirection(Speed.Y_DIRECTION_UP);
					elements.add(b);
					setActive(true);
				}
			}
			return true;
		}

	}

	public void updatePhysics() {
		if (isActive()) {
			synchronized (elements) {
				int p1width = ((Paddle) elements.get(0)).getGraphic()
						.getWidth();
				int p1height = ((Paddle) elements.get(0)).getGraphic()
						.getHeight();
				int p2width = ((Paddle) elements.get(1)).getGraphic()
						.getWidth();
				int p1xpos = ((Paddle) elements.get(0)).getCoordinates().getX();
				int p1ypos = ((Paddle) elements.get(0)).getCoordinates().getY();
				int p2xpos = ((Paddle) elements.get(1)).getCoordinates().getX();
				int p2ypos = ((Paddle) elements.get(1)).getCoordinates().getY();
				Coordinates coord;
				Speed speed;
				//TODO: init vibrator
				
				//----------------------Paddle1: controlled by user----------------------
				Paddle p1 = (Paddle) elements.get(0);
				coord = p1.getCoordinates();
				speed = p1.getSpeed();
				if (speed.getXDirection() == Speed.X_DIRECTION_RIGHT) {
					coord.setX(coord.getX() + speed.getX());
				} else {
					coord.setX(coord.getX() - speed.getX());
				}
				// Toggle x-direction on collision with borders
				if (coord.getX() < 0) {
					speed.toggleXDirection();
					coord.setX(-coord.getX());
					//TODO
				} else if (coord.getX() + p1.getGraphic().getWidth() > getWidth()) {
					speed.toggleXDirection();
					coord.setX(coord.getX() + getWidth()
							- (coord.getX() + p1.getGraphic().getWidth()));
					//TODO
				}

				//----------------------Paddle2: controlled by AI------------------------
				Paddle p2 = (Paddle) elements.get(1);
				coord = p2.getCoordinates();
				speed = p2.getSpeed();
				speed.setX(5);
				if (speed.getXDirection() == Speed.X_DIRECTION_RIGHT) {
					coord.setX(coord.getX() + speed.getX());
				} else {
					coord.setX(coord.getX() - speed.getX());
				}
				// Toggle x-direction on collision with borders
				if (coord.getX() < 0) {
					speed.toggleXDirection();
					coord.setX(-coord.getX());
				} else if (coord.getX() + p2.getGraphic().getWidth() > getWidth()) {
					speed.toggleXDirection();
					coord.setX(coord.getX() + getWidth()
							- (coord.getX() + p2.getGraphic().getWidth()));
				}

				//---------------------------Ball---------------------------------------
				Ball ball = (Ball) elements.get(2);
				coord = ball.getCoordinates();
				speed = ball.getSpeed();

				// Speed to left or right
				if (speed.getXDirection() == Speed.X_DIRECTION_RIGHT) {
					coord.setX(coord.getX() + speed.getX());
				} else {
					coord.setX(coord.getX() - speed.getX());
				}

				// Speed to up or down
				if (speed.getYDirection() == Speed.Y_DIRECTION_DOWN) {
					coord.setY(coord.getY() + speed.getY());
				} else {
					coord.setY(coord.getY() - speed.getY());
				}

				// Toggle x-direction on collision with borders
				if (coord.getX() < 0) {
					speed.toggleXDirection();
					coord.setX(-coord.getX());
					//TODO
				} else if (coord.getX() + ball.getGraphic().getWidth() > getWidth()) {
					speed.toggleXDirection();
					coord.setX(coord.getX() + getWidth()
							- (coord.getX() + ball.getGraphic().getWidth()));
					//TODO
				}

				// Toggle y-direction on collision with paddles
				if (ball instanceof Ball) {
					if (coord.getY() <= p1ypos + p1height) {
						if (coord.getX() > p1xpos
								&& coord.getX() < (p1xpos + p1width)) {
							speed.toggleYDirection();
							//TODO
							if (acceleration < 3) {
								acceleration++;
								ball.getSpeed().setY(
										ball.getSpeed().getY() + acceleration);
							}
						}
					} else if (coord.getY() + ball.getGraphic().getHeight() >= p2ypos) {
						if (coord.getX() > p2xpos
								&& coord.getX() < (p2xpos + p2width)) {
							if (acceleration < 3) {
								acceleration++;
								ball.getSpeed().setY(
										ball.getSpeed().getY() + acceleration);
							}
							speed.toggleYDirection();
							//TODO
						}
					}
				}

				// Check goal
				if (ball instanceof Ball) {
					//goal for AI
					if (coord.getY() <= 0) {
						elements.remove(ball);
						setActive(false);
						//TODO
					}
					//goal for user
					else if (coord.getY() + ball.getGraphic().getHeight() >= getHeight()) {
						elements.remove(ball);
						setActive(false);
						//TODO
					}
				}

			}
		}

	}

	@Override
	public void surfaceChanged(SurfaceHolder holder, int format, int width,
			int height) {

	}

	@Override
	public void surfaceCreated(SurfaceHolder holder) {
		initGame();
		thread.setRunning(true);
		thread.start();
	}

	@Override
	public void surfaceDestroyed(SurfaceHolder holder) {
		boolean retry = true;
		thread.setRunning(false);
		while (retry) {
			try {
				thread.join();
				retry = false;
			} catch (InterruptedException e) {
			}
		}
	}
	
	//TODO: orientationChanged

}
